package laboratoire4;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.net.Socket;

public class Client {
    private final Socket socket;
    private final BufferedInputStream input;
    private final BufferedOutputStream output;
    private PusherBoard board;

    public Client(String host, int port) throws IOException {
        socket = new Socket(host, port);
        input = new BufferedInputStream(socket.getInputStream());
        output = new BufferedOutputStream(socket.getOutputStream());
    }

    public void listen() {
        try {
            while (true) {
                char cmd;
                cmd = (char) input.read();

                if (board == null) {
                    boolean play = initializeBoard(cmd);
                    if (play) {
                        play();
                    }
                }

                if (cmd == '3') {
                    play();
                }

                if (cmd == '4') {
                    handleInvalidMove();
                }

                if (cmd == '5') {
                    stopGame();
                    break;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    private boolean initializeBoard(char cmd) throws IOException {
        Player player = cmd == '1' ? Player.RED : Player.BLACK;

        byte[] aBuffer = new byte[1024];
        int size = input.available();

        input.read(aBuffer, 0, size);
        String s = new String(aBuffer).trim();
        String[] boardValues = s.split(" ");

        board = new PusherBoard(player, boardValues);

        return player == Player.RED;
    }

    private void play() throws IOException {
        byte[] aBuffer = new byte[16];
        int size = input.available();
        if (size > 0) {
            input.read(aBuffer, 0, size);
            String previousMove = new String(aBuffer);
            System.out.println("Mouvement reçu: " + previousMove);

            board.move(previousMove);
        }

        String nextMove = board.runNextMove();
        System.out.println("Prochain mouvement: " + nextMove);

        output.write(nextMove.getBytes(), 0, nextMove.length());
        output.flush();
    }

    private void handleInvalidMove() throws InterruptedException {
        System.err.println("Mouvement invalide!");
        Thread.sleep(500);
        board.printBoard();
    }

    private void stopGame() throws IOException {
        System.out.println("GG well played!");
        socket.close();
    }
}

